// p3.h
#ifndef P3_H
#define P3_H

#include <vector>
#include <cstdint>
#include <iostream>
#include <boost/signals2.hpp>

using namespace std;

// Enumeración para representar las direcciones de movimiento del robot.
enum Direction { UP, DOWN, LEFT, RIGHT };


// -- Tipos ---------------------------------------------------------------
using obstacle_sig_t = boost::signals2::signal<void (uint32_t, uint32_t)>;
using out_map_sig_t = boost::signals2::signal<void (int32_t, int32_t)>;
using battery_sig_t = boost::signals2::signal<void (uint16_t)>;

// Funciones para obtener las señales.
obstacle_sig_t& get_obstacle_signal();
out_map_sig_t& get_out_map_signal();
battery_sig_t& get_battery_signal();


//Constantes referidas al consumo de batería para diferentes casos.
const int kBATTERY_COST_OBSTACLE = 10; //Consumo cuando se encuentra un obstáculo.
const int kBATTERY_COST_OUT_MAP = 15; //Consumo cuando se intenta salir del mapa.
const int kBATTERY_COST_MOVEMENT = 5; //Consumo cuando el robot cambia de posición tras un movimiento.


// Estructura para representar la posición de un elemento en el mapa.
struct Position {
    uint32_t row;
    uint32_t col;
};

// Estructura para representar el robot.
struct Robot {
    Position position;
    uint16_t battery;
};

// Estructura para representar un obstáculo del mapa.
struct Obstacle {
    Position pos;
};

// Estructura para representar el mapa.
struct Map {
    uint32_t rows;
    uint32_t cols;
    std::vector<Position> obstacles;

    Robot robot;
};


// Función para crear un mapa nuevo.
Map createMap(uint32_t rows, uint32_t cols, const Position &pos_robot, uint16_t battery_level=100);

// Función para colocar un obstáculo en una posición del mapa.
bool placeObstacle(Map& map, const Position& position);

// Función para verificar si una posición del mapa tiene un obstáculo.
bool isObstacle(const Map& map, const Position& position);



// Función para mover el robot en una dirección.
void moveRobot(Map& map, Direction direction);

// Función para mover el robot según una lista de instrucciones.
void moveRobotListMovements(Map &map, const vector<Direction>& list_directions);


// Función para imprimir los bordes superior e inferior del mapa.
[[maybe_unused]]
static void displayUpDownBordersMap(uint32_t cols, bool isDown)
{
    if (isDown)
        cout << "|";
    else
        cout << "___";
        
    for (uint32_t j = 0; j < cols; ++j)
        cout << "___";

    if (isDown)
        cout << "__|";
    cout << endl;
}

// Función para imprimir el mapa
[[maybe_unused]]
static void displayMap (const Map& map) {
  displayUpDownBordersMap(map.cols, false);

    for (uint32_t i = 0; i < map.rows; ++i) {
        cout << "| ";
        for (uint32_t j = 0; j < map.cols; ++j) {
            Position p = {i,j};
            if (isObstacle(map, p))
                cout << " X ";
            else
                if (map.robot.position.row == i && map.robot.position.col == j)
                    cout << " R ";
                else
                    cout << " . ";
        }
        cout << " |" << endl;
    }

    displayUpDownBordersMap(map.cols, true);
}



#endif /* P3_H */

